/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/iso/stdlib_c99.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( SOLARIS_STDLIB_NORETURN_CHECK )
extern void quick_exit(int) __attribute__((__noreturn__));
#endif  /* SOLARIS_STDLIB_NORETURN_CHECK */
