/* PR target/82880 */
/* Test we don't ICE or hang.  */
/* { dg-do compile } */
/* { dg-options "--help=target --help=optimizers" } */
/* { dg-allow-blank-lines-in-output 1 } */
/* { dg-prune-output ".*" } */
