/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/rtldef/setjmp.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( VMS_USE_FAST_SETJMP_CHECK )
#   if defined(__FAST_SETJMP) || defined (__GNUC__) || defined(__UNIX_SETJMP)
#endif  /* VMS_USE_FAST_SETJMP_CHECK */
