------------------------------------------------------------------------------
--                                                                          --
--                         GNAT COMPILER COMPONENTS                         --
--                                                                          --
--                              S C I L _ L L                               --
--                                                                          --
--                                 S p e c                                  --
--                                                                          --
--          Copyright (C) 2010-2018, Free Software Foundation, Inc.         --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 3,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.                                     --
--                                                                          --
-- As a special exception under Section 7 of GPL version 3, you are granted --
-- additional permissions described in the GCC Runtime Library Exception,   --
-- version 3.1, as published by the Free Software Foundation.               --
--                                                                          --
-- You should have received a copy of the GNU General Public License and    --
-- a copy of the GCC Runtime Library Exception along with this program;     --
-- see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see    --
-- <http://www.gnu.org/licenses/>.                                          --
--                                                                          --
-- GNAT was originally developed  by the GNAT team at  New York University. --
-- Extensive contributions were provided by Ada Core Technologies Inc.      --
--                                                                          --
------------------------------------------------------------------------------

--  This package extends the tree nodes with fields that are used to reference
--  the SCIL node and the Contract_Only_Body of a subprogram with aspects.

with Types; use Types;

package SCIL_LL is

   function Get_Contract_Only_Body (N : Node_Id) return Node_Id;
   --  Read the value of attribute Contract_Only_Body

   function Get_SCIL_Node (N : Node_Id) return Node_Id;
   --  Read the value of attribute SCIL node

   procedure Set_Contract_Only_Body (N : Node_Id; Value : Node_Id);
   --  Set the value of attribute Contract_Only_Body

   procedure Set_SCIL_Node (N : Node_Id; Value : Node_Id);
   --  Set the value of attribute SCIL node

   procedure Initialize;
   --  Initialize the table of SCIL nodes

   function Is_Contract_Only_Body (E : Entity_Id) return Boolean;
   --  Return True if E is a Contract_Only_Body subprogram

   procedure Set_Is_Contract_Only_Body (E : Entity_Id);
   --  Set E as Contract_Only_Body subprogram

end SCIL_LL;
